// =====================================================================
//  mandelbrot_host.c
// =====================================================================

//  Author:         (c) 2013 Camil Demetrescu
//  License:        See the end of this file for license information
//  Created:        June 16, 2013
 
//  Last changed:   $Date: 2013/06/16 15:00:07 $
//  Changed by:     $Author: demetres $
//  Revision:       $Revision: 1.00 $


#include "mandelbrot.h"


// ---------------------------------------------------------------------
// mandelbrot_host
// ---------------------------------------------------------------------
// sequential CPU version executed on host

void mandelbrot_host(float rc, float ic, float scale, 
                     int w, int h, int max_iter,
                     unsigned char* img, double* t) {

    // get initial time
    double start = clut_get_real_time();

    int x, y, k;
    float ip, rp, rz, iz;

    // scan image pixels
    for (y=0; y<h; y++)
        for (x=0; x<w; x++) {
            rp = rc+scale*(x-w/2); // real part of point p corresponding to pixel (x,y)
            ip = ic+scale*(y-h/2); // imaginary part of point p corresponding to pixel (x,y)
            rz = iz = 0.0;         // z = (rz, iz)
            for (k = 0; k < max_iter; k++) { // iterate z = z^2+p
                float new_iz = ip + 2.0*rz*iz;
                float new_rz = rp + rz*rz - iz*iz;
                iz = new_iz;
                rz = new_rz;
                if (rz*rz + iz*iz >= 4.0) break;
            }
            img[y*w+x] = k==max_iter ? 0 : 8*255.0*k/max_iter;
        }

    // get elapsed time
    *t = clut_get_real_time() - start;
}


// Copyright (C) 2013 Camil Demetrescu
  
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
  
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
  
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
// USA
