__kernel void mandelbrot(float rc, float ic, float scale, 
					     int w, int h, int max_iter,
					     __global unsigned char* img) {

	int x, y, k;
	float ip, rp, rz, iz;

    x = get_global_id(0);
    y = get_global_id(1);
    
    if (x>=w || y>=h) return;

	rp = rc+scale*(x-w/2);
	ip = ic+scale*(y-h/2);
	rz = iz = 0.0;
	for (k = 0; k < max_iter; k++) {
		float new_iz = ip + 2.0*rz*iz;
		float new_rz = rp + rz*rz - iz*iz;
		iz = new_iz;
		rz = new_rz;
		if (rz*rz + iz*iz >= 4.0) break;
	}
	img[y*w+x] = k==max_iter ? 0 : 8*255.0*k/max_iter;
}

